@@ -7,5 +7,5 @@ import pyrfuniverse.utils.rfuniverse_utility as utility
 
 def parse_message(msg: IncomingMessage) -> dict:
     this_object_data = {}
-    this_object_data['force'] = msg.read_float32()
+    this_object_data['forces'] = msg.read_float32_list()
     return this_object_data
